#    yum-plugin-snapper take snapshots with thy python toolkit snapper
#    in time of package changes.
#    Copyright (C) 2017  Andreas Benzler
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>


from yum.plugins import PluginYumExit, TYPE_CORE, TYPE_INTERACTIVE
# interaction with snapper through dbus
from dbus import SystemBus, Interface, DBusException

# we check root id in unix style
from os import getuid

# we use commandline to handle grub2 update
from os.path import isdir
from subprocess import Popen, PIPE

# plugin specification
requires_api_version = '2.3'
plugin_type = (TYPE_CORE, TYPE_INTERACTIVE)

# snapper dbus object
snapper = None
# hold the configs from snapper.conf
yum_snap_configs = None
# hold all active config names
snap_config_list = []
# pre ids from snapper
snap_pre_id = []
# if this is set: Only single
# snapshots are taken in the post
snap_single = None
# description for pre or single snapshots
snap_description = None

# if this is true, we do grub2 update
update_grub2 = None
# grub2 binary
grub2_cmd = "/usr/sbin/grub2-mkconfig"
# bios configuration file
grub2_bios_conf = "/etc/grub2.cfg"
# efi configuration file
grub2_efi_conf = "/etc/grub2-efi.cfg"
# path for efi check
efi_check = "/sys/firmware/efi"

#################################################################
#
# Yum plugin hooks
#
#################################################################


def init_hook(conduit):
    global snapper
    global yum_snap_configs
    global snap_single
    global update_grub2

    # only as root, otherwise we return
    if getuid()!=0:
        return

    snapper = Interface(SystemBus().get_object('org.opensuse.Snapper',
                                               '/org/opensuse/Snapper'),
                                dbus_interface='org.opensuse.Snapper')

    yum_snap_configs = conduit.confString('main', 'configs', default=None)
    snap_single = conduit.confBool('main','single', default= False)
    update_grub2 = conduit.confBool('main','update_grub2', default=False)

    # If snapper configs exist: create an array
    if yum_snap_configs:
        yum_snap_configs = yum_snap_configs.split(',')

    snapper_configs_check()

pass


def pretrans_hook(conduit):
    global yum_snap_configs
    global snap_single
    global snap_description
    
    opts, commands = conduit.getCmdLine()
    # we build a description from yum commands
    snap_description = "yum " + " ".join(commands)

    if snap_single:
        conduit.info(4, "Snapper single snapshot configured, leave pre.")
        return

    do_presnapshots(conduit)

pass


def posttrans_hook(conduit):

    do_postsnapshots(conduit)

    if update_grub2:
        do_update_grub2(conduit)

pass

#################################################################
#
# Snapper functions
#
#################################################################


def snapper_configs_check():
    global yum_snap_configs
    global snap_config_list
    global snapper

    try:

        found_root = False

        user_snapper_configs = snapper.ListConfigs()
        # we need any kind of configuration
        if not user_snapper_configs:
            raise PluginYumExit("Snapper: Is not configured!")

        # No configuration inside of snapper.conf
        # search for a root level config
        if not yum_snap_configs:
            for cfg in user_snapper_configs:
                # found root level ?
                if cfg[1] == '/':
                    snap_config_list.append(cfg[0])
                    found_root = True
        else:
            cfg_ar = []
            root_config = None
            # create a single array, more easy for check
            for cfg in user_snapper_configs:
                cfg_ar.append(cfg[0])
                # found root level ?
                if cfg[1] == '/':
                    root_config = cfg[0]

            for cfg in yum_snap_configs:
                # configuration exist for snapper.conf -> configs ??
                if not cfg in cfg_ar:
                    raise PluginYumExit("Snapper: No configuration found for '" +
                                        cfg + "'! Check [main] variable 'configs' ! ")
                else:
                    snap_config_list.append(cfg)
                    # is one of the give entries root level?
                    if cfg == root_config:
                        found_root=true

        if not found_root:
            raise PluginYumExit("Snapper: No configuration found for root element in filesystem!")

    except DBusException as e:
        raise PluginYumExit("Snapper: connect to snapperd failed " + e + "!")

pass

def do_presnapshots(conduit):

    global snapper
    global snap_description
    global snap_config_list
    global snap_single
    global snap_pre_id

    try:
        for cfg in snap_config_list:
            pre_id = snapper.CreatePreSnapshot(cfg, snap_description, 'number', {})
            snap_pre_id.append(pre_id)
            conduit.info(4, "Snapper pre: id '" + str(pre_id) + '" configuration '"" + cfg + "'.")
    except DBusException as e:
        raise PluginYumExit("Snapper: fails on pre snapshot - " + e + "!")

pass


def do_postsnapshots(conduit):

    global snapper
    # we use snapper pre list array for post or single
    global snap_config_list
    # we need the pre id's for post snapshot
    global snap_pre_id
    global snap_description
    global snap_single

    try:
        i=0
        for cfg in snap_config_list:
            if snap_single:
                single_id = snapper.CreateSingleSnapshot(cfg, snap_description, 'number', {})
                conduit.info(4, "Snapper single: id '" + str(single_id) + '" configuration '"" + cfg + "'.")
            else:
                post_id = snapper.CreatePostSnapshot(cfg, str(snap_pre_id[i]), '', 'number', {})
                conduit.info(4, "Snapper post: id '" + str(post_id) + '" configuration '"" + cfg + "'.")
                i+=1

    except DBusException as e:
        raise PluginYumExit("Snapper: fails on post snapshot - " + e + "!")

pass

#################################################################
#
# Update grub2
#
#################################################################


def do_update_grub2(conduit):
    if isdir(efi_check):
        config_file = grub2_efi_conf
    else:
        config_file = grub2_bios_conf

    p = Popen([grub2_cmd, '--output', config_file],stdout=PIPE,stderr=PIPE)
    output, errors = p.communicate()

    # grub2 output is written into errors
    # only show up at debug level 4
    conduit.info(4,errors)

pass